/*
 * @Descripttion: 
 * @version: 
 * @Author: dingjian
 * @Date: 2021-03-17 18:52:07
 * @LastEditors: dingjian
 * @LastEditTime: 2021-05-05 11:34:40
 */
// import 'dart:html';
import 'dart:io';
import 'dart:convert';
import 'package:analyzer/dart/analysis/features.dart';
import 'package:analyzer/dart/analysis/utilities.dart';
import 'package:args/args.dart';
import 'package:yz_dsl_gen/visitor/yingzi_ast_visitor.dart';

import 'json_utils.dart';

void genAst(List<String> arguments) async {
  exitCode = 0; // presume success
  final parser = ArgParser()..addFlag('file', negatable: false, abbr: 'f');

  var argResults = parser.parse(arguments);

  stdout.writeln(argResults.rest);
  final paths = argResults.rest;
  if (paths.isEmpty) {
    stdout.writeln('No file found');
  } else {
    var newRootPath = await writeFile(paths[0]);
    stdout.writeln(newRootPath);

    var _results = conversionToYzJson(await generate(paths[0]));
    // _selectedRsult = null;
    if (_results != null && _results.isNotEmpty) {
      _results.forEach((result) {
        var newFile = File(newRootPath + result.functionName + '.json');
        newFile.writeAsStringSync(result.resultJsonString);
      });
    }
  }
}

Future<String> writeFile(String path) async {
  var file = Directory(path).absolute;
  var originPath = file.path;
  var destinationRootPath = originPath.replaceRange(
      originPath.lastIndexOf('/'), originPath.length, '/yingzi_ast/');

  var newDirectory = Directory(destinationRootPath);
  try {
    var exists = await newDirectory.exists();
    if (!exists) {
      await newDirectory.create();
    }
  } catch (e) {
    print(e);
  }
  return newDirectory.path;
}

//生成AST
Future generate(String path) async {
  if (path.isEmpty) {
    stdout.writeln('No file found');
  } else {
    await _handleError(path);
    if (exitCode == 2) {
      try {
        var parseResult =
            parseFile(path: path, featureSet: FeatureSet.fromEnableFlags([]));
        var compilationUnit = parseResult.unit;
        //遍历AST
        var astData = compilationUnit.accept(YZWidgetAstVisitor());
        stdout.writeln(jsonEncode(astData));
        return Future.value(jsonEncode(astData));
      } catch (e) {
        stdout.writeln('Parse file error: ${e.toString()}');
      }
    }
  }
  return Future.value();
}

Future _handleError(String path) async {
  if (await FileSystemEntity.isDirectory(path)) {
    stderr.writeln('error: $path is a directory');
  } else {
    exitCode = 2;
  }
}
