/*
* @Author: yz.dingjian
* @Description: 
* @Date: 2021/3/9 11:21 AM
* @Last Modified by: yz.dingjian
* @Last Modified time: 2021/3/9 11:21 AM
*/

import 'dart:convert';

/// **********

/// 解析ast生成的json，转换成适用于yingzi的格式
///
// {
//   'name':'functionName',
//   'body':'resultString'
// }
List<ResultModel> conversionToYzJson(String orignJson) {
  List list = jsonDecode(orignJson)['root'];
  List<ResultModel> modelList = [];
  list.forEach((element) {
    String resultJson = jsonEncode(conversionFunction(element['body']));
    modelList.add(ResultModel(
        functionName: element['name'], resultJsonString: resultJson));
  });
  return modelList;
}

// 解析转换body 方法
Map conversionFunction(Map widgetMap) {
  Map newTotalMap = {};
  // 遍历widgetMap
  widgetMap.forEach((key, value) {
    if (key == 'widgetName') {
      newTotalMap[key] = value;
    }
    if (key == 'props') {
      newTotalMap[key] = conversionProps(value, widgetMap['widgetName']);
    }
    // {
    //   "type": "StringLiteral",
    //   "value": "选1"
    // }
    if (key == 'value') {
      newTotalMap['value'] = "$value";
    }
  });
  return newTotalMap;
}

Map conversionProps(Map props, String className) {
  List propList = props['propsList'];
  String yzType = props['yzType'];
  Map tempMap = {};
  List dataList = [];
  propList.forEach((element) {
    // element 有 name expression 时有子节点 继续解析
    // 仅有 type 和 value 时为叶子节点
    Map expressionMap = element['expression'];
    Map nameMap = element['name'];
    String type = element['type'];
    var value = element['value'];

    if (yzType != null) {
      tempMap['type'] = yzType;
    }

    if (value != null) {
      /// [Text]
      if (className == 'Text') {
        tempMap['data'] = value;

        /// [BorderRadius]
      } else if (className == 'BorderRadius') {
        if (yzType == 'circular') {
          tempMap['radius'] = '$value';
        }

        /// [Radius]
      } else if (className == 'Radius') {
        if (yzType == 'circular') {
          tempMap['radius'] = '$value';
        }

        /// [Image]
      } else if (className == 'Image') {
        if (yzType != null && yzType == 'asset') {
          tempMap['name'] = '$value';
        } else if (yzType != null && yzType == 'network') {
          tempMap['src'] = '$value';
        }
      } else {
        tempMap['dataList'] = dataList;
        dataList.add(value);
      }
    }

    if (expressionMap == null) return {};

    // 被调用者名称
    Map callee = expressionMap['callee'];
    String calleeName = '';
    if (callee != null) {
      Map calleeObject = callee['object'];
      if (calleeObject != null) {
        calleeName = calleeObject['name'];
      }
    }

    if (expressionMap['widgetName'] != null) {
      // 这里解析具体方法，
      /// 具体方法具体解析，如[padding]
      Map propsMap = expressionMap['props'];
      // if (expressionMap['widgetName'] == 'fromLTRB') {
      //   tempMap[nameMap['name']] = conversionPadding(propsMap);
      // }

      if (calleeName != null && calleeName == 'EdgeInsets') {
        tempMap[nameMap['name']] = conversionPadding(propsMap);
      }

      /// [color] 特殊处理
      else if (nameMap['name'] == 'color' &&
          expressionMap['widgetName'] == 'Color') {
        tempMap['color'] = conversionColor(propsMap);
      }

      /// [AssetImage NetworkImage] 特殊处理
      else if (expressionMap['widgetName'] == 'NetworkImage' ||
          expressionMap['widgetName'] == 'AssetImage') {
        tempMap[nameMap['name']] =
            conversionImageProvider(propsMap, expressionMap['widgetName']);
      }

      /// [style]
      else if (nameMap['name'] == 'style' &&
          expressionMap['widgetName'] == 'TextStyle') {
        tempMap['style'] = conversionStyle(propsMap);
      } else {
        /// 一般情况
        tempMap[nameMap['name']] = {
          'widgetName': expressionMap['widgetName'],
          'props': conversionProps(propsMap, expressionMap['widgetName'])
        };
      }
    }
    // 参数是列表
    else if (expressionMap['list'] != null) {
      List funcList = expressionMap['list'];
      if (nameMap['name'] == 'children') {}

      List resultList = [];

      /// 如果参数是方法，则继续解析
      /// 如果参数是基本数据类型，那么直接赋值
      /// 约定 如果是数据，需要转成json字符串

      if (funcList.length > 0) {
        if (funcList.first['type'] == 'StringLiteral') {
          /// 如果是 基本数据类型的 list，必须将list转为字符串
          funcList.forEach((element) {
            resultList.add('${element['value']}');
          });
          tempMap[nameMap['name']] = jsonEncode(resultList);
        } else {
          funcList.forEach((element) {
            resultList.add(conversionFunction(element));
          });
          tempMap[nameMap['name']] = resultList;
        }
      }
    } else if (expressionMap['type'] != null) {
      if (expressionMap['type'] == 'PrefixedIdentifier') {
        // tempMap['value'] = expressionMap['identifier']['name'];
        // tempMap['prefix'] = expressionMap['prefix']['name'];
        tempMap[nameMap['name']] = expressionMap['identifier']['name'];
      }

      /// 当expressionMap为value时
      // {
      //   "name": {
      //     "type": "Identifier",
      //     "name": "height"
      //   },
      //   "expression": {
      //     "type": "IntegerLiteral",
      //     "value": 200
      //   }
      // }
      else if (expressionMap['type'] == 'StringLiteral' ||
          expressionMap['type'] == 'IntegerLiteral' ||
          expressionMap['type'] == 'StringLiteral') {
        tempMap[nameMap['name']] = '${expressionMap['value']}';
      }
    }
  });
  return tempMap;
}

/* 转换前格式
    {
      "type": "propsList",
      "propsList": [
        {
          "type": "IntegerLiteral",
          "value": 2
        },
        {
          "type": "IntegerLiteral",
          "value": 3
        },
        {
          "type": "IntegerLiteral",
          "value": 4
        },
        {
          "type": "IntegerLiteral",
          "value": 5
        }
      ]
    }
   */

/* 转换后格式
    '[2,3,4,5]'
   */

String conversionPadding(Map originMap) {
  List propsList = originMap['propsList'];
  String resultString = '';
  for (var i = 0; i < propsList.length; i++) {
    String elementString = '${propsList[i]['value']}';
    if (i == 0) {
      resultString = '[' + elementString;
    } else if (i == propsList.length - 1) {
      resultString = resultString + ',' + elementString + ']';
    } else {
      resultString = resultString + ',' + elementString;
    }
  }
  return resultString;
}

/* 转换前格式
    {
      "type": "propsList",
      "propsList": [
        {
          "type": "IntegerLiteral",
          "value": 4279383126
        }
      ]
    }
   */

/* 转换后格式
    '0x2345456'
   */
String conversionColor(Map originMap) {
  List propsList = originMap['propsList'];
  String typeName = propsList.first['type'];
  if (typeName == 'IntegerLiteral') {
    int value = propsList.first['value'];
    String resultString = '0x${value.toRadixString(16)}';
    return resultString;
  }

  return '';
}

/* 转换前格式
    {
      "type": "propsList",
      "propsList": [
        {
          "name": {
            "type": "Identifier",
            "name": "fontWeight"
          },
          "expression": {
            "type": "PrefixedIdentifier",
            "identifier": {
              "type": "Identifier",
              "name": "bold"
            },
            "prefix": {
              "type": "Identifier",
              "name": "FontWeight"
            }
          }
        },
        {
          "name": {
            "type": "Identifier",
            "name": "color"
          },
          "expression": {
            "functionName": "Color",
            "props": {
              "type": "propsList",
              "propsList": [
                {
                  "type": "IntegerLiteral",
                  "value": "1193046"
                }
              ]
            },
            "callee": {
              "type": "Identifier",
              "name": "Color"
            }
          }
        },
        {
          "name": {
            "type": "Identifier",
            "name": "fontSize"
          },
          "expression": {
            "type": "IntegerLiteral",
            "value": "24"
          }
        }
      ]
    }
   */

/* 转换后格式
    {
      "fontWeight": "bold",
      "color": "0x123456",
      "fontSize": "12"
    }
   */
Map conversionStyle(Map originMap) {
  List propsList = originMap['propsList'];
  Map resultMap = {};
  propsList.forEach((e) {
    Map expressionMap = e['expression'];
    Map nameMap = e['name'];
    if (nameMap['name'] == 'fontWeight') {
      Map valueIdentifier = expressionMap['identifier'];
      String valueName = valueIdentifier['name'];
      resultMap[nameMap['name']] = valueName;
    } else if (nameMap['name'] == 'color') {
      Map propsMap = expressionMap['props'];
      resultMap['color'] = conversionColor(propsMap);
    } else if (nameMap['name'] == 'fontSize') {
      resultMap['fontSize'] = '${expressionMap['value']}';
    }
  });

  return resultMap;
}

/* ImageProvider转换
转换前格式
    {
    "widgetName": "AssetImage",
    "props": {
      "type": "propsList",
      "propsList": [
        {
          "type": "StringLiteral",
          "value": "images/icon_alarm_level_middle.png"
        },
        {
          "name": {
            "type": "Identifier",
            "name": "package"
          },
          "expression": {
            "type": "StringLiteral",
            "value": "woshi package"
          }
        }
      ]
    }
   */

/* 
  转换后格式
    {
      "runtimeType": "AssetImage",
      "assetName": "0x123456",(AssetImage)
      "package": "12",(AssetImage)
      "url":"",(NetworkImage)
      "scale":""(NetworkImage)
    }
   */
Map conversionImageProvider(Map originMap, String className) {
  List propsList = originMap['propsList'];
  Map resultMap = {};
  propsList.forEach((element) {
    var value = element['value'];
    var name = element['name'];
    var expression = element['expression'];
    resultMap['runtimeType'] = className;
    if (value != null) {
      /// [AssetImage]
      if (className == 'AssetImage') {
        resultMap['assetName'] = value;

        /// [NetworkImage]
      } else if (className == 'NetworkImage') {
        resultMap['url'] = value;
      }
    }

    if (name != null && expression != null) {
      resultMap[name['name']] = '${expression['value']}';
    }
  });

  return resultMap;
}

class ResultModel {
  String functionName;
  String resultJsonString;

  ResultModel({this.functionName, this.resultJsonString});

  ResultModel.fromJson(Map<String, dynamic> json) {
    functionName = json['name'];
    resultJsonString = json['body'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['name'] = this.functionName;
    data['body'] = this.resultJsonString;
    return data;
  }
}
